package com.usky.system.controller.web;


import com.ruoyi.common.core.constant.Constants;
import com.ruoyi.common.core.exception.job.TaskException;
import com.ruoyi.common.core.utils.StringUtils;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.log.annotation.Log;
import com.usky.common.log.enums.BusinessType;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.system.domain.SysJob;
import com.usky.system.service.SysJobService;
import com.usky.system.service.util.CronUtils;
import com.usky.system.service.util.ScheduleUtils;
import org.quartz.SchedulerException;
import org.springframework.beans.factory.BeanExpressionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 定时任务调度表 前端控制器
 * </p>
 *
 * @author JCB
 * @since 2022-11-18
 */
@RestController
@RequestMapping("/sysJob")
public class SysJobController {

    @Autowired
    private SysJobService jobService;

    /**
     * @param jobName      任务名称
     * @param jobGroup     任务组名
     * @param status       状态（0正常 1暂停）
     * @param invokeTarget 调用目标字符串
     * @param id           任务ID
     * @param pageNum      当前页
     * @param pageSize     每页条数
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<SysJob>> list(@RequestParam(value = "jobName", required = false) String jobName,
                                              @RequestParam(value = "jobGroup", required = false) String jobGroup,
                                              @RequestParam(value = "status", required = false) String status,
                                              @RequestParam(value = "invokeTarget", required = false) String invokeTarget,
                                              @RequestParam(value = "id", required = false, defaultValue = "0") Integer id,
                                              @RequestParam(value = "pageNum", required = false, defaultValue = "1") Integer pageNum,
                                              @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize) {
        return ApiResult.success(jobService.selectJobList(jobName, jobGroup, status, invokeTarget, id, pageNum, pageSize));
    }

    /**
     * 定时任务-新增
     */
    @Log(title = "定时任务-新增", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public ApiResult<Void> add(@RequestBody SysJob job) throws SchedulerException, TaskException {
        if (!CronUtils.isValid(job.getCronExpression())) {
            throw new BeanExpressionException("新增任务'" + job.getJobName() + "'失败，Cron表达式不正确");
        } else if (StringUtils.containsIgnoreCase(job.getInvokeTarget(), Constants.LOOKUP_RMI)) {
            throw new BeanExpressionException("新增任务'" + job.getJobName() + "'失败，目标字符串不允许'rmi'调用");
        } else if (StringUtils.containsAnyIgnoreCase(job.getInvokeTarget(), new String[]{Constants.LOOKUP_LDAP, Constants.LOOKUP_LDAPS})) {
            throw new BeanExpressionException("新增任务'" + job.getJobName() + "'失败，目标字符串不允许'ldap(s)'调用");
        } else if (StringUtils.containsAnyIgnoreCase(job.getInvokeTarget(), new String[]{Constants.HTTP, Constants.HTTPS})) {
            throw new BeanExpressionException("新增任务'" + job.getJobName() + "'失败，目标字符串不允许'http(s)'调用");
        } else if (StringUtils.containsAnyIgnoreCase(job.getInvokeTarget(), Constants.JOB_ERROR_STR)) {
            throw new BeanExpressionException("新增任务'" + job.getJobName() + "'失败，目标字符串存在违规");
        } else if (!ScheduleUtils.whiteList(job.getInvokeTarget())) {
            throw new BeanExpressionException("新增任务'" + job.getJobName() + "'失败，目标字符串不在白名单内");
        }
        job.setCreateBy(SecurityUtils.getUsername());
        jobService.insertJob(job);
        return ApiResult.success();
    }

    /**
     * 定时任务-修改
     */
    @Log(title = "定时任务-修改", businessType = BusinessType.UPDATE)
    @PutMapping("edit")
    public ApiResult<Void> edit(@RequestBody SysJob job) throws SchedulerException, TaskException {
        if (!CronUtils.isValid(job.getCronExpression())) {
            throw new BeanExpressionException("修改任务'" + job.getJobName() + "'失败，Cron表达式不正确");
        } else if (StringUtils.containsIgnoreCase(job.getInvokeTarget(), Constants.LOOKUP_RMI)) {
            throw new BeanExpressionException("修改任务'" + job.getJobName() + "'失败，目标字符串不允许'rmi'调用");
        } else if (StringUtils.containsAnyIgnoreCase(job.getInvokeTarget(), new String[]{Constants.LOOKUP_LDAP, Constants.LOOKUP_LDAPS})) {
            throw new BeanExpressionException("修改任务'" + job.getJobName() + "'失败，目标字符串不允许'ldap(s)'调用");
        } else if (StringUtils.containsAnyIgnoreCase(job.getInvokeTarget(), new String[]{Constants.HTTP, Constants.HTTPS})) {
            throw new BeanExpressionException("修改任务'" + job.getJobName() + "'失败，目标字符串不允许'http(s)'调用");
        } else if (StringUtils.containsAnyIgnoreCase(job.getInvokeTarget(), Constants.JOB_ERROR_STR)) {
            throw new BeanExpressionException("修改任务'" + job.getJobName() + "'失败，目标字符串存在违规");
        } else if (!ScheduleUtils.whiteList(job.getInvokeTarget())) {
            throw new BeanExpressionException("修改任务'" + job.getJobName() + "'失败，目标字符串不在白名单内");
        }
        job.setUpdateBy(SecurityUtils.getUsername());
        jobService.updateJob(job);
        return ApiResult.success();
    }

    /**
     * 定时任务-状态修改
     */
    @Log(title = "定时任务-状态修改", businessType = BusinessType.UPDATE)
    @PutMapping("/changeStatus")
    public ApiResult<Void> changeStatus(@RequestBody SysJob job) throws SchedulerException {
        SysJob newJob = jobService.selectJobById(job.getJobId());
        newJob.setStatus(job.getStatus());
        jobService.changeStatus(newJob);
        return ApiResult.success();
    }

    /**
     * 定时任务-立即执行一次
     */
    @Log(title = "定时任务-立即执行一次", businessType = BusinessType.UPDATE)
    @PutMapping("/run")
    public ApiResult<Void> run(@RequestBody SysJob job) throws SchedulerException {
        jobService.run(job);
        return ApiResult.success();
    }

    /**
     * 定时任务-删除
     */
    @Log(title = "定时任务-删除", businessType = BusinessType.DELETE)
    @DeleteMapping("/remove")
    public ApiResult<Void> remove(@RequestParam(value = "jobId") long jobId) throws SchedulerException, TaskException {
        jobService.deleteJobByIds(jobId);
        return ApiResult.success();
    }

}

