package com.usky.system.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.log.annotation.Log;
import com.usky.common.log.enums.BusinessType;
import com.usky.system.domain.SysJobLog;
import com.usky.system.service.SysJobLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 定时任务调度日志表 前端控制器
 * </p>
 *
 * @author JCB
 * @since 2022-11-18
 */
@Controller
@RequestMapping("/sysJobLog")
public class SysJobLogController {
    @Autowired
    private SysJobLogService jobLogService;

    /**
     * 查询定时任务调度日志列表
     *
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<SysJobLog>> list(@RequestParam(value = "jobName", required = false) String jobName,
                                                 @RequestParam(value = "jobGroup", required = false) String jobGroup,
                                                 @RequestParam(value = "status", required = false) String status,
                                                 @RequestParam(value = "invokeTarget", required = false) String invokeTarget,
                                                 @RequestParam(value = "id", required = false, defaultValue = "0") Integer id,
                                                 @RequestParam(value = "pageNum", required = false, defaultValue = "1") Integer pageNum,
                                                 @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize) {
        return ApiResult.success(jobLogService.selectJobLogList(jobName, jobGroup, status, invokeTarget, id, pageNum, pageSize));
    }

    /**
     * 根据调度编号获取详细信息
     */
    @GetMapping(value = "/{configId}")
    public ApiResult<SysJobLog> getInfo(@PathVariable Long jobLogId) {
        return ApiResult.success(jobLogService.selectJobLogById(jobLogId));
    }

    /**
     * 删除定时任务调度日志
     */
    @Log(title = "定时任务调度日志", businessType = BusinessType.DELETE)
    @DeleteMapping("/{jobLogIds}")
    public ApiResult<Void> remove(@PathVariable Long[] jobLogIds) {
        jobLogService.deleteJobLogByIds(jobLogIds);
        return ApiResult.success();
    }

    /**
     * 清空定时任务调度日志
     */
    @Log(title = "调度日志", businessType = BusinessType.CLEAN)
    @DeleteMapping("/clean")
    public ApiResult<Void> clean() {
        jobLogService.cleanJobLog();
        return ApiResult.success();
    }

}

