package com.usky.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.system.domain.SysJobLog;
import com.usky.system.mapper.SysJobLogMapper;
import com.usky.system.service.SysJobLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 * 定时任务调度日志表 服务实现类
 * </p>
 *
 * @author JCB
 * @since 2022-11-18
 */
@Service
public class SysJobLogServiceImpl extends AbstractCrudService<SysJobLogMapper, SysJobLog> implements SysJobLogService {

    @Autowired
    private SysJobLogMapper jobLogMapper;

    /**
     * 获取quartz调度器日志的计划任务
     *
     * @param jobName
     * @param jobGroup
     * @param status
     * @param invokeTarget
     * @param id
     * @param pageNum
     * @param pageSize
     * @return
     */
    public CommonPage<SysJobLog> selectJobLogList(String jobName, String jobGroup, String status, String invokeTarget, Integer id, Integer pageNum, Integer pageSize) {
        IPage<SysJobLog> page = new Page<>(pageNum, pageSize);
        LambdaQueryWrapper<SysJobLog> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(StringUtils.isNotBlank(jobName), SysJobLog::getJobName, jobName)
                .eq(StringUtils.isNotBlank(jobGroup), SysJobLog::getJobGroup, jobGroup)
                .eq(StringUtils.isNotBlank(status), SysJobLog::getStatus, status)
                .like(StringUtils.isNotBlank(invokeTarget), SysJobLog::getInvokeTarget, invokeTarget)
                .eq(id != null && id != 0, SysJobLog::getJobLogId, id);
        page = this.page(page, queryWrapper);
        return new CommonPage<>(page.getRecords(), page.getTotal(), pageSize, pageNum);
    }

    /**
     * 通过调度任务日志ID查询调度信息
     *
     * @param jobLogId 调度任务日志ID
     * @return 调度任务日志对象信息
     */
    @Override
    public SysJobLog selectJobLogById(Long jobLogId) {
        LambdaQueryWrapper<SysJobLog> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(SysJobLog::getJobLogId, jobLogId);
        SysJobLog list = this.getOne(queryWrapper);
        return list;
    }

    /**
     * 新增任务日志
     *
     * @param jobLog 调度日志信息
     */
    @Override
    public void addJobLog(SysJobLog jobLog) {
        jobLog.setCreateTime(LocalDateTime.now());
        this.save(jobLog);
    }

    /**
     * 批量删除调度日志信息
     *
     * @param logIds 需要删除的数据ID
     * @return 结果
     */
    @Override
    public void deleteJobLogByIds(Long[] logIds) {
        for (int i = 0; i < logIds.length; i++) {
            this.removeById(logIds[i]);
        }
    }

    /**
     * 删除任务日志
     *
     * @param jobId 调度日志ID
     */
    @Override
    public int deleteJobLogById(Long jobId) {
        boolean b = this.removeById(jobId);
        int row = 0;
        if (b) {
            row = 1;
        }
        return row;
    }

    /**
     * 清空任务日志
     */
    @Override
    public void cleanJobLog() {
        jobLogMapper.cleanJobLog();
    }

}
